@@ -14,9 +14,11 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import configureoutput
+import buildinfo
 import errno, sys
 import locale
+import tempfile
+import shutil
 import soundcard, mpd
 import lessonfile
 import dataparser
@@ -28,6 +30,7 @@ import gu
 from osutils import *
 import reportlib
 import filesystem
+import osutils
 
 def check_rcfile():
     """See default.config for rcfileversion values, meanings and
@@ -100,6 +103,9 @@ class SolfegeApp(cfg.ConfigUtils):
         options -- command line options parsed by optparse
         """
         cfg.ConfigUtils.__init__(self, 'solfege-app')
+        self.locate_external_programs()
+        lessonfile.MusicBaseClass.temp_dir = tempfile.mkdtemp(prefix="solfege-")
+        os.environ['SOLFEGETEMPDIR'] = lessonfile.MusicBaseClass.temp_dir
         # test_mode is when we are running a test from the Tests menu
         self.m_test_mode = False
         self.m_options = options
@@ -109,6 +115,20 @@ class SolfegeApp(cfg.ConfigUtils):
         self.lessonfile_manager = lessonfile_manager
         self.m_sound_init_exception = None
         self.setup_sound()
+    def locate_external_programs(self):
+        """
+        This method check if the user has configured the path to some
+        programs we need. Set the config variable if not.
+        This is useful the first time Solfege is run.
+        """
+        if not os.path.exists(cfg.get_string("programs/csound")):
+            v = osutils.find_progs(("csound", "csound.exe"))
+            if v:
+                self.set_string("programs/csound", v[0])
+        if not os.path.exists(cfg.get_string("programs/mma")):
+            v = osutils.find_mma_executables()
+            if v:
+                self.set_string("programs/mma", v[0])
     def setup_sound(self):
         if sys.platform == 'win32' and \
                     cfg.get_string("sound/type") == "sequencer-device":
@@ -146,7 +166,7 @@ class SolfegeApp(cfg.ConfigUtils):
             self.m_ui.display_error_message(
 _("You should configure sound from the 'Sound' page of the preferences window."))
         elif cfg.get_string("sound/type") == "sequencer-device":
-            if not configureoutput.HAVE_LINUX_AWE_VOICE_H and \
+            if not buildinfo.HAVE_LINUX_AWE_VOICE_H and \
                     cfg.get_string("sound/card_info") == "awe":
                 cfg.set_string("sound/card_info", "");
             try:
@@ -333,11 +353,15 @@ It is also possible that the OS sound setup is incorrect.
             self.m_ui.box_dict[self.m_ui.m_viewer].on_end_practise()
             self.m_ui.box_dict[self.m_ui.m_viewer].on_start_practise()
     def quit_program(self):
-        cfg.sync()
+        try:
+            cfg.sync()
+        except IOError, e:
+            self.m_ui.display_exception_message(e)
         if self.m_ui.m_viewer != 'docviewer':
             self.m_ui.box_dict[self.m_ui.m_viewer].on_end_practise()
         if soundcard.synth:
             soundcard.synth.close()
+        shutil.rmtree(lessonfile.MusicBaseClass.temp_dir, True)
     def export_training_set(self, export_data, export_dir, output_format):
         """
         This function requires a program that can create WAV files
@@ -543,42 +567,4 @@ It is also possible that the OS sound setup is incorrect.
                     teacher.m_tonika.get_octave_notename(),
                     (teacher.m_tonika + teacher.m_interval).get_octave_notename())
                 yield ret
-    def win32_get_langenviron(self):
-        """
-        Return the language defined in filesystem.app_data()/langenviron.bat
-        win32 only
-        """
-        assert sys.platform == 'win32'
-        s = ""
-        try:
-            langfile = open(os.path.join(filesystem.app_data(), "langenviron.bat"), 'rU')
-            s = langfile.read()
-            langfile.close()
-        except IOError, e:
-            # we get here for example when the file does not exist
-            return 'system default'
-        if s:
-            r = re.compile("set LANGUAGE=(?P<lang>.*)")
-            for line in s.split("\n"):
-                m = r.match(line)
-                if m:
-                    return m.groups()[0]
-        return 'system default'
-    def win32_put_langenviron(self, lang):
-        """
-        Write the filesystem.app_data()/langenviron.bat file.
-        win32 only
-        """
-        try:
-            langfile = open(os.path.join(filesystem.app_data(), "langenviron.bat"), 'w')
-            print >> langfile, "rem Created by GNU Solfege %s" % configureoutput.VERSION_STRING
-            if lang:
-                print >> langfile, "set LANGUAGE=%s" % lang
-            else:
-                print >> langfile, "rem System default language. Not setting variable."
-            langfile.close()
-        except IOError, e:
-            self.m_app.m_ui.display_exception_message(e)
-        m = self.win32_get_langenviron()
-        print "GOT", m
 
