@@ -18,7 +18,7 @@ import sys
 import os
 import gobject, gtk
 import gu, mpd
-import configureoutput
+import buildinfo
 import notenamespinbutton
 from instrumentselector import nInstrumentSelector, InstrumentConfigurator
 import cfg
@@ -26,6 +26,7 @@ import osutils
 import soundcard
 import languages
 import filesystem
+import winlang
 
 if sys.platform == 'win32':
     try:
@@ -35,21 +36,6 @@ if sys.platform == 'win32':
         print e
         winmidi = None
 
-def find_wav_player_programs():
-    ret = []
-    for p, cmd in (('/usr/bin/play', '/usr/bin/play %s'),
-                   ('/usr/bin/aplay', '/usr/bin/aplay %s'),
-                   ('/usr/bin/esdplay', '/usr/bin/esdplay %s')):
-        if os.path.exists(p):
-            ret.append(cmd)
-    return ret
-
-def find_midi_player_programs():
-    for p, cmd in (('/usr/bin/timidity', '/usr/bin/timidity -idqq %s'),
-                   ('/usr/bin/drvmidi', '/usr/bin/drvmidi'),
-                   ('/usr/bin/playmidi', '/usr/bin/playmidi')):
-        if os.path.exists(p):
-            yield cmd
 
 class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
     def on_destroy(self, widget, e):
@@ -195,12 +181,37 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             self.g_ogg_convertor, sizegroup))
 
         self.add_gui_for_external_programs(page_vbox)
+        ########
+        # Misc #
+        ########
+        sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
 
         box, category_vbox = gu.hig_category_vbox(_("Misc"))
         page_vbox.pack_start(box, False)
 
+        # CSound
         liststore = gtk.ListStore(gobject.TYPE_STRING)
-        for n in ("xterm -e mutt", "thunderbird"):
+        for p in osutils.find_progs(('csound', 'csound.exe')):
+            liststore.append((p,))
+        self.g_csound_exe = gtk.ComboBoxEntry(liststore)
+        self.g_csound_exe.child.set_text(self.get_string("programs/csound"))
+        self.g_csound_exe.connect('changed', lambda w: self.set_string('programs/csound', self.g_csound_exe.child.get_text()))
+        box = gu.hig_label_widget(_("CSound executable:"), self.g_csound_exe, sizegroup)
+        category_vbox.pack_start(box)
+        # MMA
+        liststore = gtk.ListStore(gobject.TYPE_STRING)
+        for p in osutils.find_mma_executables():
+            liststore.append((p,))
+        self.g_mma_exe = gtk.ComboBoxEntry(liststore)
+        self.g_mma_exe.child.set_text(self.get_string("programs/mma"))
+        self.g_mma_exe.connect('changed', lambda w: self.set_string('programs/mma', self.g_mma_exe.child.get_text()))
+        box = gu.hig_label_widget(_("MMA executable:"), self.g_mma_exe, sizegroup)
+        category_vbox.pack_start(box)
+        # Mail program
+        liststore = gtk.ListStore(gobject.TYPE_STRING)
+        for n in osutils.find_progs(('mutt', 'thunderbird', 'evolution')):
+            if os.path.split(n)[1] == 'mutt':
+                n = "xterm -e %s" % n
             liststore.append((n,))
         self.g_mail_program = gtk.ComboBoxEntry(liststore)
         self.g_mail_program.child.set_text(self.get_string("config/mua"))
@@ -217,6 +228,9 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         box, category_vbox = gu.hig_category_vbox(_("Gui"))
         page_vbox.pack_start(box, False)
 
+        b = gu.nCheckButton('gui', 'web_browser_as_help_browser',
+            _("Use a _web browser as help browser"))
+        category_vbox.pack_start(b, False)
         self.g_mainwin_user_resizeable = gu.nCheckButton('gui',
           'mainwin_user_resizeable', _("_User resizeable main window"))
         category_vbox.pack_start(self.g_mainwin_user_resizeable, False)
@@ -232,7 +246,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             self.g_language.append_text(n)
         label.set_mnemonic_widget(self.g_language)
         if sys.platform == 'win32':
-            lang = self.m_app.win32_get_langenviron()
+            lang = winlang.win32_get_langenviron()
             if lang in languages.languages:
                 idx = languages.languages.index(lang)
             else:
@@ -248,9 +262,9 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             if sys.platform == 'win32':
                 if combobox.get_active():
                     lang = languages.languages[combobox.get_active()]
-                    app.win32_put_langenviron(lang)
+                    winlang.win32_put_langenviron(lang)
                 else:
-                    app.win32_put_langenviron(None)
+                    winlang.win32_put_langenviron(None)
         self.g_language.connect_after('changed', f)
         hbox.pack_start(self.g_language, False)
         category_vbox.pack_start(hbox)
@@ -295,12 +309,9 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
                  # testfile is a file in lesson-files/share
                  'testfile': 'fifth-small-220.00.wav',
                  'label': _("WAV file player"),
-                 # players is a tuple of tuples. The inner tuple has to elements:
+                 # players is a list of tuples. The tuple has to elements:
                  # '/path/to/player', 'comandline'
-                 'players': (('/usr/bin/play', '/usr/bin/play %s'),
-                             ('/usr/bin/aplay', '/usr/bin/aplay %s'),
-                             ('/usr/bin/esdplay', '/usr/bin/esdplay %s'),
-                            ),
+                 'players': [],
                 }
             ]
         else:
@@ -309,33 +320,36 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             {'id': 'midi',
              'testfile': 'fanfare.midi',
              'label': _("MIDI file player"),
-             'players': (('/usr/bin/timidity', '/usr/bin/timidity -idqq %s'),
-                         ('/usr/bin/drvmidi', '/usr/bin/drvmidi'),
-                         ('/usr/bin/playmidi', '/usr/bin/playmidi'),
-                        ),
+             'players': [],
             },
             {'id': 'mp3',
              'testfile': 'fanfare.mp3',
              'label': _("MP3 file player"),
-             'players': (('/usr/bin/mpg123', '/usr/bin/mpg123'),
-                         ('/usr/bin/alsaplayer', '/usr/bin/alsaplayer'),
-                        ),
+             'players': [],
             },
             {'id': 'ogg',
              'testfile': 'fanfare.ogg',
              'label': _("OGG file player"),
-             'players': (('/usr/bin/ogg123', '/usr/bin/ogg123'),
-                         ('/usr/bin/alsaplayer', '/usr/bin/alsaplayer'),
-                        ),
+             'players': [],
             },
         ])
+        for d in format_info:
+            for bin, opts in {
+                'wav': (('play', '%s'), ('aplay', '%s'), ('esdplay', '%s')),
+                'midi': (('timidity', '-idqq %s'),
+                         ('drvmidi', ''),
+                         ('playmidi', '')),
+                'mp3': (('mpg123', ''), ('alsaplayer', '')),
+                'ogg': (('ogg123', ''), ('alsaplayer', ''))}[d['id']]:
+                v = osutils.find_progs((bin,))
+                if v:
+                    d['players'].append((v[0], " ".join((v[0], opts))))
         self.g_player_boxes = {}
         sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
         for format in format_info:
             liststore = gtk.ListStore(gobject.TYPE_STRING)
             for path, cmd in format['players']:
-                if os.path.exists(path):
-                    liststore.append((cmd,))
+                liststore.append((cmd,))
             self.g_player_boxes[format['id']] = gtk.ComboBoxEntry(liststore)
             self.g_player_boxes[format['id']].child.set_text(cfg.get_string('sound/%s_player' % format['id']))
             def _changed(widget, formatid):
@@ -351,8 +365,12 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
                                       sizegroup)
             category_vbox.pack_start(box)
     def test_XXX_player(self, w, typeid, testfile):
-        osutils.run_external_program(self.g_player_boxes[typeid].child.get_text(),
-                 'lesson-files/share', testfile)
+        try:
+            osutils.run_external_program(
+                self.g_player_boxes[typeid].child.get_text(),
+                'lesson-files/share', testfile)
+        except Exception, e:
+            self.m_app.m_ui.display_exception_message(e)
     def create_linux_sound_page(self):
         page_vbox = gu.hig_dlg_vbox()
         self.g_notebook.append_page(page_vbox, gtk.Label(_("Sound setup")))
@@ -387,7 +405,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         hbox.pack_start(gtk.Label("    "))
         self.g_awe_checkbutton = gtk.CheckButton(_("_My sound card is Sound Blaster AWE32, AWE64 or pnp32"))
         self.g_awe_checkbutton.set_active(self.get_string('sound/card_info') == 'awe')
-        if not configureoutput.HAVE_LINUX_AWE_VOICE_H:
+        if not buildinfo.HAVE_LINUX_AWE_VOICE_H:
             self.g_awe_checkbutton.set_sensitive(False)
         else:
             self.g_awe_checkbutton.set_sensitive(self.get_string("sound/type") == "sequencer-device")
@@ -505,7 +523,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
                   self.g_player_boxes['midi'].child.get_text())
             soundcard.synth.error_report_cb = self.m_app.m_ui.display_error_message
         elif self.g_device_radio.get_active():
-            if sys.platform != 'win32' and configureoutput.HAVE_LINUX_AWE_VOICE_H and \
+            if sys.platform != 'win32' and buildinfo.HAVE_LINUX_AWE_VOICE_H and \
                 self.g_awe_checkbutton.get_active():
                 card_info = "awe"
             else:
