@@ -1,4 +1,5 @@
 # GNU Solfege - free ear training software
+# vim: set fileencoding=utf-8 :
 # Copyright (C) 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
@@ -235,7 +236,7 @@ class FlashBar(gtk.Frame):
             gobject.source_remove(self.__timeout)
             self.__timeout = None
         self.__stack.append(txt)
-        self.__label.set_text(txt, '')
+        self.__label.set_text(txt)
     def pop(self):
         """If a message is being flashed right now, that flashing is
         not affected, but the message below the flashed message is removed.
@@ -347,15 +348,20 @@ class HarmonicProgressionLabel(gtk.HBox):
     This class can parse strings like I-(6,4)V(5,3)-I and can be used
     as button labels.
     """
-    def __init__(self, str, align=''):
+    def __init__(self, str):
         gtk.HBox.__init__(self)
         self.show()
-        self.set_text(str, align)
-    def set_text(self, str, align=''):
+        self.m_xalign = 0.0
+        self.m_yalign = 0.5
+        self.set_text(str)
+    def set_alignment(self, xalign, yalign):
+        self.m_xalign = xalign
+        self.m_yalign = yalign
+    def set_text(self, str):
         for o in self.get_children():
             o.destroy()
         self.m_str = str
-        if align == 'center':
+        if self.m_xalign == 0.5:
             self.pack_start(gtk.HBox())
         while self.m_str:
             T, A, B = self.get_next_token()
@@ -366,7 +372,7 @@ class HarmonicProgressionLabel(gtk.HBox):
             else:
                 assert T == 'one'
                 self.oneline(A)
-        if align == 'center':
+        if self.m_xalign == 0.5:
             self.pack_start(gtk.HBox())
         self.show_all()
     def get_next_token(self):
@@ -412,6 +418,7 @@ class HarmonicProgressionLabel(gtk.HBox):
         t1.show()
         self.pack_start(t1, False)
 
+
 def dialog_yesno(text, parent=None):
     """Return True if the answer is yes, False if the answer is no.
     """
