@@ -27,6 +27,35 @@ import lessonfilegui
 from gpath import Path
 import filesystem
 
+def get_learning_tree_list(debug):
+    """
+    Return a list of tuples with info about the learning trees found.
+    Tuple objects:
+    1. The string "user" or "solfege"
+    2. The filename, not including the name of the directory
+    3. Full path to the learning tree.
+    """
+    trees = []
+    for cls, dir in (('solfege', 'learningtrees'),
+        ('user', os.path.join(filesystem.user_data(), "learningtrees"))):
+        try:
+            v = os.listdir(dir)
+        except OSError:
+            v = []
+        for fn in v:
+            if cls == 'solfege':
+                fullpath = os.path.join("learningtrees", fn)
+            else:
+                fullpath = os.path.join(filesystem.app_data(), "learningtrees", fn)
+            trees.append((cls, fn, fullpath))
+    if not debug:
+        try:
+            trees.remove(('solfege', 'debugtree.txt', os.path.join('solfege', 'debugtree.txt')))
+        except ValueError:
+            # The debugtree.txtfile is for some reason missing
+            pass
+    return trees
+
 class Menu(dict):
     def new_from_name(name):
         m = Menu({'name': name, 'children': []})
@@ -68,12 +97,26 @@ class LearningTree:
         self.m_menus = []
         self.m_deps = {}
         self.m_modified = False
+        self.m_title = ''
+        self.m_learning_tree_version = 3
+        self.m_visibility = 0
     def load(self, filename):
+        self.read_file(filename)
+        self.post_load()
+    def read_file(self, filename):
         self.m_visibility = 0
         self.m_learning_tree_version = 0
         g = {'_': lambda s: s}
         v = eval(open(filename, 'rU').read(), g)
-        if len(v) == 2:
+        if type(v) == dict:
+            self.m_menus = v['menu']
+            self.m_deps = v['deps']
+            self.m_title = v.get('title', '')
+            self.m_visibility = v['visibility']
+            self.m_learning_tree_version = v['version']
+        # The different lengths of v below is to be able to read
+        # old formats of the learning tree.
+        elif len(v) == 2:
             self.m_menus, self.m_deps = v
         elif len(v) == 3:
             self.m_menus, self.m_deps, self.m_visibility = v
@@ -89,6 +132,9 @@ class LearningTree:
             ]
             self.m_menus[0]['children'] = [{'name': n['name'], 'children': n['lessons']} for n in self.m_menus[0]['submenus']]
             del self.m_menus[0]['submenus']
+        if not self.m_title:
+            self.m_title = filename
+    def post_load(self):
         self.m_menus = [Menu(d) for d in self.m_menus]
         for idx, menu in enumerate(self.m_menus):
             self.m_menus[idx].children = [Menu(s) for s in menu.children]
@@ -117,10 +163,15 @@ class LearningTree:
         self.calculate_visibilities()
         self.m_modified = False
     def save(self, filename):
+        self.m_learning_tree_version = 3
         ofile = open(filename, 'w')
-        print >> ofile, "[ # toplevel list"
-        print >> ofile, "  %s, # learning tree file format version" % self.m_learning_tree_version
-        print >> ofile, "  [ # start of list of menus"
+        print >> ofile, "# GNU Solfege learning tree"
+        print >> ofile, "{"
+        print >> ofile, " 'version': %i," % self.m_learning_tree_version
+        print >> ofile, " 'title': '%s'," % self.m_title.replace("'", r"\'")
+        print >> ofile, " 'titletotranslate': _('%s')," % self.m_title.replace("'", r"\'")
+        print >> ofile, " 'visibility': %i," % self.m_visibility
+        print >> ofile, " 'menu': [ # start of list of menus"
         for menu in self.m_menus:
             print >> ofile, menu.pformat(4)
         print >> ofile, "  ], # end of list of menus"
@@ -129,7 +180,10 @@ class LearningTree:
         lesson_ids = list(self.iterate_all_lessons())
         for dep in [d for d in self.m_deps if d not in lesson_ids]:
             del self.m_deps[dep]
-        ofile.write("%s,\n%s]" % (pprint.pformat(self.m_deps), self.m_visibility))
+        print >> ofile, " 'deps':",
+        print >> ofile, "%s," % pprint.pformat(self.m_deps)
+        print >> ofile, "# End of dict mappding dependencies."
+        print >> ofile, "}"
         ofile.close()
         self.m_modified = False
     def new_menu(self, menuname):
@@ -397,6 +451,15 @@ class LearningTree:
         if id_b in deps_b:
             return 1
         return cmp(len(deps_b), len(deps_a))
+    def get_use_dict(self):
+        """
+        return a dict mapping lesson_id to the number of times the lesson
+        has been added to the learning tree.
+        """
+        retval = {}
+        for lesson_id in self.iterate_all_lessons():
+            retval[lesson_id] = retval.get(lesson_id, 0) + 1
+        return retval
     def get_use_count(self, lesson_id):
         """
         Return an integer telling how many times the lesson lesson_id
@@ -437,7 +500,6 @@ class LessonFileDialogCommon(gtk.Dialog):
             (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,))
         self.m_app = app
         self.m_tree = tree
-        self.m_app.lessonfile_manager.parse(self.m_app.m_options.debug)
         lessonfilegui.handle_lesson_id_crash(self.m_app.lessonfile_manager)
         self.m_cur_menu_path = path
         self.g_ok_button = self.add_button("gtk-ok", gtk.RESPONSE_OK)
@@ -518,20 +580,37 @@ class LessonFileDialogCommon(gtk.Dialog):
 class SelectLessonFileDialog(LessonFileDialogCommon):
     def __init__(self, app, tree, path):
         LessonFileDialogCommon.__init__(self, app, tree, path)
+        hbox = gtk.HBox()
+        hbox.show()
+        self.vbox.pack_start(hbox, False)
+        self.vbox.reorder_child(hbox, 0)
         g_show_used = gu.nCheckButton('SelectLessonFileDialog',
             'show_used_files', _("_Show files used in other topics"),
             False, self.update_filelist)
-        self.vbox.pack_start(g_show_used, False)
-        self.vbox.reorder_child(g_show_used, 0)
+        hbox.pack_start(g_show_used, False)
+        b = gtk.Button(_("_Search for new lesson files"))
+        b.connect('clicked', self.search_for_lesson_files)
+        b.show()
+        hbox.pack_start(b, False)
+    def search_for_lesson_files(self, btn):
+        self.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
+        # Had to run main_iteration to make the cursor change.
+        while gtk.events_pending():
+            gtk.main_iteration()
+        self.m_app.lessonfile_manager.parse(self.m_app.m_options.debug)
+        lessonfilegui.handle_lesson_id_crash(self.m_app.lessonfile_manager)
+        self.update_filelist()
+        self.window.set_cursor(None)
     def update_filelist(self, w=None):
         self.g_liststore.clear()
+        use_count = self.m_tree.get_use_dict()
         if cfg.get_bool('SelectLessonFileDialog/show_used_files'):
             lesson_id_list = [i for i in list(self.m_app.lessonfile_manager.iterate_lesson_ids()) if i not in self.m_tree.get(self.m_cur_menu_path).children]
         else:
-            lesson_id_list = [i for i in list(self.m_app.lessonfile_manager.iterate_lesson_ids()) if self.m_tree.get_use_count(i) == 0]
+            lesson_id_list = [i for i in list(self.m_app.lessonfile_manager.iterate_lesson_ids()) if use_count.get(i, 0) == 0]
         for lesson_id in lesson_id_list:
             d = self.m_app.lessonfile_manager.m_uiddb[lesson_id]
-            self.g_liststore.append((d['header']['title'], d['filename'], lesson_id, self.m_tree.get_use_count(lesson_id), self.m_app.lessonfile_manager.get(lesson_id, 'module')))
+            self.g_liststore.append((d['header']['title'], d['filename'], lesson_id, use_count.get(lesson_id, 0), self.m_app.lessonfile_manager.get(lesson_id, 'module')))
 
 
 class DepsLessonFileDialog(LessonFileDialogCommon):
@@ -587,6 +666,15 @@ class Window(gtk.Window):
         self.g_content_vbox.set_border_width(12)
         self.g_content_vbox.set_spacing(8)
         buttons_sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
+        # Title
+        hbox = gtk.HBox()
+        hbox.set_spacing(8)
+        l = gtk.Label(_("Title:"))
+        hbox.pack_start(l, False)
+        self.g_title = gtk.Entry()
+        self.g_title.connect('changed', self.on_title_entry_changed)
+        hbox.pack_start(self.g_title, False)
+        self.g_content_vbox.pack_start(hbox, False)
         # Visibility
         hbox = gtk.HBox()
         hbox.set_spacing(8)
@@ -701,18 +789,10 @@ class Window(gtk.Window):
         self.treeview_cursor_changed()
         self.__load_file_lock = 0
     def fill_trees_combo(self):
-        self.m_trees = [('solfege', 'learningtree.txt', os.path.join(os.getcwdu(), 'learningtree.txt'))]
-        if self.m_app.m_options.debug:
-            self.m_trees.append(('solfege', 'debugtree.txt', os.path.join(os.getcwdu(), 'debugtree.txt')))
-        try:
-            v = os.listdir(os.path.join(filesystem.app_data(), "learningtrees"))
-        except OSError:
-            v = []
-        for fn in v:
-            self.m_trees.append(('user', fn, os.path.join(filesystem.app_data(), "learningtrees", fn)))
+        self.m_trees = get_learning_tree_list(self.m_app.m_options.debug)
         self.g_trees_liststore.clear()
-        for t, fn, txt in self.m_trees:
-            self.g_trees_liststore.append((txt,))
+        for cls, fn, fullpath in self.m_trees:
+            self.g_trees_liststore.append((fullpath,))
     def set_trees_combo(self, place, filename):
         # Set the combo to the correct learning tree.
         self.__load_file_lock += 1
@@ -724,7 +804,7 @@ class Window(gtk.Window):
             idx += 1
         self.__load_file_lock -= 1
     def on_learning_tree_combo_changed(self, combobox):
-        t, fn, txt = self.m_trees[combobox.get_active()]
+        t, fn, fullpath = self.m_trees[combobox.get_active()]
         if self.m_tree.m_modified:
             if gu.dialog_yesno(_("The file is not saved. Save before changing?"), self):
                 self._do_save(self.m_filename)
@@ -892,6 +972,8 @@ class Window(gtk.Window):
     def update_gui(self):
         self.__create_model()
         self.set_title(self.m_filename)
+    def on_title_entry_changed(self, entry):
+        self.m_tree.m_title = entry.get_text()
     def on_visibility_entry_changed(self, *v):
         try:
             if self.g_visibility.get_text():
@@ -1188,14 +1270,14 @@ class Window(gtk.Window):
         This function will also make sure the g_trees combo is set.
         """
         if place == 'solfege':
-            self._load_file(filename)
+            self._load_file(os.path.join("learningtrees", filename))
         else:
             assert place == 'user'
             self._load_file(os.path.join(filesystem.app_data(), 'learningtrees', filename))
         self.set_trees_combo(place, filename)
         # 
         if place == 'solfege':
-            if not os.access(filename, os.W_OK):
+            if not os.access(os.path.join("learningtrees", filename), os.W_OK):
                 m = gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO,
                     gtk.BUTTONS_CLOSE, _("The default learning tree is write protected in your install. This is normal. If you want to edit a learning tree, you have to select one of the trees stored in .solfege/learningtrees in your home directory."))
                 m.run()
@@ -1207,6 +1289,7 @@ class Window(gtk.Window):
         self.m_filename = filename
         self.m_tree.load(filename)
         self.g_visibility.set_text(str(self.m_tree.m_visibility))
+        self.g_title.set_text(self.m_tree.m_title)
         self._m_orig_deps = copy.deepcopy(self.m_tree.m_deps)
         self._m_orig_menus = copy.deepcopy(self.m_tree.m_menus)
         self._m_orig_visibility = self.m_tree.m_visibility
