@@ -1,4 +1,5 @@
 # GNU Solfege - free ear training software
+# vim: set fileencoding=utf-8 :
 # Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
@@ -16,6 +17,7 @@
 
 
 import errno
+import tempfile
 import textwrap
 import glob
 import xrandom
@@ -71,9 +73,6 @@ class NoQuestionsConfiguredException(LessonfileException):
             _("You can select questions on the config page of the exercise."))
 
 
-class plstring(unicode):
-    pass
-
 # The keys in the dict say how many steps up or down in the
 # circle of fifths we go if we transpose.
 _keys_to_interval = {
@@ -135,6 +134,7 @@ keywords = (
     'cmdline',
     'wavfile',
     'midifile',
+    'mma',
     'horiz',
     'vertic',
     'accidentals',
@@ -165,26 +165,89 @@ predef = {
 for n in keywords:
     predef[n] = n
 
+def load_text(parser, filename):
+    f = open(os.path.join(parser.m_location, filename), 'r')
+    s = f.read()
+    f.close()
+    return s
+
+class nrandom(object):
+    def __init__(self, v):
+        if len(v) == 1 and type(v[0]) == list:
+            v = v[0]
+        self.m_list = v
+        self.randomize()
+    def __str__(self):
+        return self.m_list[self.m_idx]
+    def randomize(self):
+        self.m_idx = random.randint(0, len(self.m_list) - 1)
+
+class prandom(nrandom):
+    """
+    Every call to __str__ will randomize.
+    """
+    def __str__(self):
+        self.randomize()
+        return self.m_list[self.m_idx]
+
+class LabelObject(object):
+    def __init__(self, labeltype, labeldata):
+        self.m_labeltype = labeltype
+        self.m_labeldata = labeldata
+    def __getattr__(self, name):
+        if name == 'cval':
+            return (self.m_labeltype, self.m_labeldata)
+
 lessonfile_functions = {
-        '_': dataparser.dataparser_i18n_func,
-        '_i': dataparser.dataparser_i18n__i_func,
+           #  (need_parser, function)
+        '_': (False, dataparser.dataparser_i18n_func),
+        '_i': (False, dataparser.dataparser_i18n__i_func),
+        'load': (True, load_text),
+        'nrandom': (False, lambda *v: nrandom(v)),
+        'prandom': (False, lambda *v: prandom(v)),
         # play_wav should probably be removed. Replaced by wavfile
-        'play_wav': lambda f: Wavfile(f),
-        'music': lambda m: Music(m),
-        'chord': lambda m: Chord(m),
-        'satb': lambda m: Satb(m),
-        'voice': lambda m: Voice(m),
-        'rvoice': lambda m: Rvoice(m),
-        'rhythm': lambda m: Rhythm(m),
-        'percussion': lambda m: Percussion(m),
-        'cmdline': lambda m: Cmdline(m),
-        'wavfile': lambda m: Wavfile(m),
-        'midifile': lambda m: Midifile(m),
-        'mp3file': lambda m: Mp3file(m),
-        'oggfile': lambda m: Oggfile(m),
-        'progressionlabel': lambda s: plstring(s),
+        'play_wav': (False, lambda f: Wavfile(f)),
+        'music': (False, lambda m: Music(m)),
+        'chord': (False, lambda m: Chord(m)),
+        'satb': (False, lambda m: Satb(m)),
+        'voice': (False, lambda m: Voice(m)),
+        'rvoice': (False, lambda m: Rvoice(m)),
+        'rhythm': (False, lambda m: Rhythm(m)),
+        'percussion': (False, lambda m: Percussion(m)),
+        'cmdline': (False, lambda m: Cmdline(m)),
+        'csound': (True, lambda parser, orc, sco: CSound(parser, orc, sco)),
+        'wavfile': (False, lambda m: Wavfile(m)),
+        'midifile': (False, lambda m: Midifile(m)),
+        'mp3file': (False, lambda m: Mp3file(m)),
+        'oggfile': (False, lambda m: Oggfile(m)),
+        'mma': (False, lambda *v: Mma(*v)),
+        'progressionlabel': (False, lambda s: LabelObject("progressionlabel", s)),
+        'pangomarkup': (False, lambda s: LabelObject("pangomarkup", s)),
+        'plabel': (False, lambda a, b, c: LabelObject("plabel", (a, b, c))),
+        'rnc': (False, lambda s: LabelObject("rnc", s)),
 }
 
+
+def rn_markup_tokenizer(s):
+    """
+    [rn][mod1][num][\s-]
+    """
+    rn_re = re.compile("""(?P<p1>b?[ivIV]+)
+                          (?P<p2>[^\d\s-]*)
+                          (?P<p3>[^\s-]*)
+                          (?P<sep>(\s*-\s*|\s*))""",
+                       re.VERBOSE)
+    i = 0
+    retval = []
+    while True:
+        m = rn_re.match(s[i:])
+        if not m:
+            break
+        retval.append((m.group('p1'), m.group('p2'), m.group('p3'), m.group('sep')))
+        i += m.end()
+    return retval
+
+
 class _Header(dict):
     def __init__(self, headerdict):
         dict.__init__(self, headerdict)
@@ -222,6 +285,16 @@ class _Header(dict):
 
 
 class MusicBaseClass(object):
+    """
+    MusicBaseClass.temp_dir should be set by the application before
+    we create any instances. Any subclasses of MusicBaseClass that need
+    to write temporary files, should write then to this directory.
+
+    Music types that want to do some randomization for each new
+    question should implement a .randomize method. One type that does
+    this is Mma.
+    """
+    temp_dir = None
     def __init__(self, musicdata):
         self.m_musicdata = musicdata
     def get_mpd_music_string(self, lessonfile_ref):
@@ -299,6 +372,11 @@ class Chord(ChordCommon):
                 % (lessonfile_ref.m_transpose.get_octave_notename(),
                    self.m_musicdata)
         return "\\staff{<\n%s\n>}" % self.m_musicdata
+    def strip_trailing_digits(self, s):
+        # Used by get_music_as_notename_list
+        while '0' <= s[-1] <= '9':
+            s = s[:-1]
+        return s
     def get_music_as_notename_list(self, lessonfile_ref):
         """
         This method will validate the notenames, and raise a
@@ -306,11 +384,14 @@ class Chord(ChordCommon):
         m_linepos1, m_linepos2 og m_lineno.
         """
         assert isinstance(lessonfile_ref, LessonfileCommon)
+        # We strip the trailing digits to allow chords with length modifiers,
+        # like chord("c'2 e").
+        notenames = [self.strip_trailing_digits(n) for n in self.m_musicdata.split()]
         try:
             if not lessonfile_ref.header.random_transpose:
-                return [mpd.MusicalPitch.new_from_notename(n).get_octave_notename() for n in self.m_musicdata.split()]
+                return [mpd.MusicalPitch.new_from_notename(n).get_octave_notename() for n in notenames]
             else:
-                return [mpd.MusicalPitch.new_from_notename(n).transpose_by_musicalpitch(lessonfile_ref.m_transpose).get_octave_notename() for n in self.m_musicdata.split()]
+                return [mpd.MusicalPitch.new_from_notename(n).transpose_by_musicalpitch(lessonfile_ref.m_transpose).get_octave_notename() for n in notenames]
         except mpd.InvalidNotenameException, e:
             e.m_lineno, e.m_linepos1, e.m_linepos2 = mpdutils.validate_only_notenames(self.m_musicdata)
             raise
@@ -590,6 +671,48 @@ class Midifile(_MusicExternalPlayer):
     def __init__(self, musicdata):
         _MusicExternalPlayer.__init__(self, 'midi', musicdata)
 
+class Mma(MusicBaseClass):
+    def __init__(self, *v):
+        """
+        Mma is constructed in one of two ways:
+            Mma(mmacode)
+            Mma(groove, mmacode)
+        If constructed with one argument, the mmacode must be complete and
+        valid code that mma will accept.
+
+        Constructed with two arguments, the first is the name of the groove
+        to use, and the second is mma code that does not contain a Groove
+        statement. This is useful if the groove argument is a nrandom
+        or prandom object.
+        """
+        if len(v) == 1:
+            self.m_groove = None
+            mmacode = v[0]
+        else:
+            assert len(v) == 2
+            self.m_groove = v[0]
+            mmacode = v[1]
+        MusicBaseClass.__init__(self, mmacode)
+    def randomize(self):
+        try:
+            if self.m_groove:
+               self.m_groove.randomize()
+        except AttributeError:
+            pass
+    def play(self, lessonfile_ref, question):
+        infile = os.path.join(self.temp_dir, 'f.mma')
+        outfile = os.path.join(self.temp_dir, 'f.mid')
+        f = open(infile, "w")
+        if self.m_groove:
+            print >> f, "Groove %s" % self.m_groove
+        if lessonfile_ref.header.random_transpose:
+            print >> f, "Transpose %i" % (lessonfile_ref.m_transpose.semitone_pitch() - mpd.MusicalPitch.new_from_notename("c'").semitone_pitch())
+        f.write(self.m_musicdata)
+        f.close()
+        mma = cfg.get_string("programs/mma")
+        if os.system("%(mma)s -f %(outfile)s %(infile)s" % locals()) != 0:
+            raise osutils.RunningExecutableFailed(mma)
+        soundcard.play_mediafile('midi', os.path.join(self.temp_dir, 'f.mid'))
 
 class Wavfile(_MusicExternalPlayer):
     def __init__(self, musicdata):
@@ -611,6 +734,24 @@ class Cmdline(MusicBaseClass):
         osutils.run_external_program(str(self.m_musicdata),
                                      lessonfile_ref.m_location, "")
 
+class CSound(MusicBaseClass):
+    def __init__(self, parser, orc, sco):
+        MusicBaseClass.__init__(self, (orc, sco))
+    def play(self, lessonfile_ref, question):
+        outfile = open(os.path.join(self.temp_dir, "in.orc"), 'w')
+        outfile.write(self.m_musicdata[0])
+        outfile.close()
+        outfile = open(os.path.join(self.temp_dir, "in.sco"), 'w')
+        outfile.write(self.m_musicdata[1])
+        outfile.close()
+        if not os.path.exists(cfg.get_string("programs/csound")):
+            raise osutils.ExecutableDoesNotExist(cfg.get_string("programs/csound"))
+        os.system(cfg.get_string("programs/csound") + " %s %s -W -d -o %s" % (
+            os.path.join(self.temp_dir, "in.orc"),
+            os.path.join(self.temp_dir, "in.sco"),
+            os.path.join(self.temp_dir, "out.wav")))
+        soundcard.play_mediafile('wav', os.path.join(self.temp_dir, "out.wav"))
+
 
 class Music(MpdDisplayable):
     def __init__(self, musicdata):
@@ -776,6 +917,10 @@ class QuestionsLessonfile(LessonfileCommon):
                 and (len(self.m_questions) > 1 or self.header.random_transpose):
                 continue
             break
+        try:
+            self.get_question().music.randomize()
+        except AttributeError, e:
+            pass
         self.m_random.add(self._idx)
         self.m_prev_question = self.get_music()
     def find_random_transpose(self):
@@ -1002,6 +1147,10 @@ class QuestionsLessonfile(LessonfileCommon):
                 continue
             else:
                 self.m_questions.append(question)
+    def implements_music_displayer_stafflines(self):
+        if 'music_displayer_stafflines' not in self.header:
+            self.header.music_displayer_stafflines = 1
+
 
 class TestSupport(object):
     """
@@ -1137,7 +1286,14 @@ class IdByNameLessonfile(QuestionsLessonfile, TestSupport):
         # Also, if some questions has cuemusic, then we need the displayer
         if [q for q in self.m_questions if 'cuemusic' in q]:
             self.header.have_music_displayer = True
+        # FIXME to be backward compatible. FIXMECOMPAT
+        # These 3 lines make the idbyname lesson header assignment
+        # "labelformat = progression" do as advertised.
+        if self.header.labelformat == 'progression':
+            for question in self.m_questions:
+                question.label = lessonfile_functions["progressionlabel"][1](question.name)
         self.discard_questions_without_name()
+        self.implements_music_displayer_stafflines()
 
 class SingAnswerLessonfile(QuestionsLessonfile):
     def parse_string(self, s, really_filename=None):
@@ -1261,6 +1417,23 @@ class ElembuilderLessonfile(QuestionsLessonfile):
         super(ElembuilderLessonfile, self).parse_string(s, really_filename)
         # We need the name variable for statistics
         self.discard_questions_without_name()
+        # This loop let us use normal strings and lesson file functions
+        # like plabel and pangomarkup as element labels.
+        for question in self.m_questions:
+            if [e for e in question['elements'] if type(e) in (istr, LabelObject)]:
+                v = []
+                for e in question['elements']:
+                    if type(e) == istr:
+                        v.append({'name': e.cval, 'label': e.cval})
+                    elif type(e) == LabelObject:
+                        v.append({'name':e, 'label': e})
+                    else:
+                        v.append(e)
+                question['elements'] = v
+        self.implements_music_displayer_stafflines()
+        if 'element' not in self.blocklists:
+            self.blocklists['element'] = []
+
 
 
 class LessonFileManager:
@@ -1275,7 +1448,6 @@ class LessonFileManager:
         if debug:
             lessonpath.append('regression-lesson-files')
         self.m_uiddb = {}
-        vim_tmpfile_re = re.compile("\..*\.swp")
         for dir in lessonpath:
             if not os.path.isdir(dir):
                 print "warning: invalid directory in path:", dir
@@ -1285,6 +1457,7 @@ class LessonFileManager:
                 filename = filename.decode(locale.getpreferredencoding())
                 # since I usually run solfege from the source dir:
                 if os.path.split(filename)[-1] in ('.arch-ids', 'Makefile', 'Makefile.in') \
+                  or filename.endswith("~") \
                   or not os.path.isfile(filename):
                     continue
                 # We save the returned lesson_id because we need it below
@@ -1429,6 +1602,7 @@ def parse_lesson_file_header(filename):
     s = open(filename, 'rU').read()
     m = r.search(s)
     p = dataparser.Dataparser(predef, lessonfile_functions)
+    p.m_location = os.path.split(filename)
     p.m_ignore_lookup_error = True
     if not m:
         return
