@@ -1,4 +1,5 @@
 # GNU Solfege - free ear training software
+# vim: set fileencoding=utf-8 :
 # Copyright (C) 2006, 2007, 2008 Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
@@ -76,3 +77,40 @@ def handle_lesson_id_crash(lessonfile_manager):
         dlg.destroy()
 
 
+def rn_markup(s):
+    v = []
+    for p1, p2, p3, sep in lessonfile.rn_markup_tokenizer(s):
+        v.append('<span font_family="serif"><span size="xx-large">%s</span><span size="small">%s</span><span size="x-large">%s%s</span></span>' % (p1, p2, p3, sep))
+    return "".join(v)
+
+
+def new_labelobject(label):
+    """
+    label is a tuple (labeltype, labelstring) or a unicode string
+    Return a gtk widget of some sort that displays the label.
+    """
+    if isinstance(label, unicode):
+        l = gtk.Label(label)
+        l.show()
+    else:
+        if isinstance(label, tuple):
+            labeltype, labeldata = label
+        else:
+            labeltype = label.m_labeltype
+            labeldata = label.m_labeldata
+        if labeltype == 'progressionlabel':
+            l = gu.HarmonicProgressionLabel(labeldata)
+            l.show_all()
+        else:
+            l = gtk.Label()
+            if labeltype == 'rnc':
+                l.set_markup(rn_markup(labeldata))
+            elif labeltype == 'plabel':
+                l.set_markup("""<span font_family="serif"><span size="xx-large">%s</span><span size="small">%s</span><span size="x-large">%s</span></span>""" % labeldata)
+
+            elif labeltype == 'pangomarkup':
+                l.set_markup(labeldata)
+            l.show()
+    return l
+
+
