@@ -15,6 +15,16 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import webbrowser
+# We move x-www-browser to the end of the list because on my
+# debian etch system, the browser does will freeze solfege until
+# I close the browser window.
+try:
+    i = webbrowser._tryorder.index("x-www-browser")
+    webbrowser._tryorder.append(webbrowser._tryorder[i])
+    del webbrowser._tryorder[i]
+except ValueError:
+    pass
 
 import sys
 import traceback
@@ -23,6 +33,8 @@ import time
 import urllib
 import shutil
 
+import winlang
+
 # The sets module was added in python 2.3. So we will use that module
 # if we are running python 2.3, and the builtin set if we are using
 # python 2.4
@@ -36,7 +48,7 @@ except NameError:
 
 import runtime
 
-import configureoutput
+import buildinfo
 
 solfege_copyright = u"Copyright  1999-2008 Tom Cato Amundsen <tca@gnu.org>, and others."
 
@@ -103,17 +115,23 @@ opt_parser.add_option('--show-gtk-warnings', action='store_true',
 options, args = opt_parser.parse_args()
 
 if options.version:
-    print """GNU Solfege %s
+    if buildinfo.is_release():
+        rev_info = ""
+    else:
+        rev_info = u"\n    " + u"\n    ".join(buildinfo.get_bzr_revision_info_list())
+    print (u"""GNU Solfege %s%s
 This is free software. It is covered by the GNU General Public License,
 and you are welcome to change it and/or distribute copies of it under
 certain conditions. Invoke as `solfege --warranty` for more information.
 
 %s
-        """ % (configureoutput.VERSION_STRING, solfege_copyright.encode(sys.getfilesystemencoding(), 'replace'))
+        """ % (buildinfo.VERSION_STRING,
+        rev_info,
+        solfege_copyright)).encode(sys.getfilesystemencoding(), 'replace')
     sys.exit()
 
 if options.warranty:
-    print "GNU Solfege %s" % configureoutput.VERSION_STRING
+    print "GNU Solfege %s" % buildinfo.VERSION_STRING
     print solfege_copyright.encode(sys.getfilesystemencoding(), 'replace')
     print warranty
     sys.exit()
@@ -123,6 +141,7 @@ if options.warranty:
 runtime.init(options)
 
 import gtk
+import utils
 
 class SplashWin(gtk.Window):
     def __init__(self):
@@ -135,7 +154,7 @@ class SplashWin(gtk.Window):
         vbox = gtk.VBox()
         vbox.set_border_width(20)
         frame.add(vbox)
-        l = gtk.Label(_("Starting GNU Solfege %s") % configureoutput.VERSION_STRING)
+        l = gtk.Label(_("Starting GNU Solfege %s") % buildinfo.VERSION_STRING)
         l.set_name("Heading1")
         vbox.pack_start(l)
         l = gtk.Label("http://www.solfege.org")
@@ -167,13 +186,11 @@ import tracebackwindow
 sys.stderr = tracebackwindow.TracebackWindow(options.show_gtk_warnings)
 
 import app
-import utils
 from configwindow import ConfigWindow
 import mpd
 import mpd.musicdisplayer
 import gu
 import cfg
-import stock
 import lessonfile
 import lessonfilegui
 import abstract
@@ -231,7 +248,6 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         self._vbox = gtk.VBox()
         self._vbox.show()
         self.add(self._vbox)
-        self.icons = stock.SolfegeIconFactory(self, datadir)
         self.set_default_size(400, 400)
         pixbuf = self.render_icon('solfege-icon', gtk.ICON_SIZE_DIALOG)
         self.set_icon(pixbuf)
@@ -252,7 +268,7 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         self.m_app = app.SolfegeApp(options, self, lessonfile_manager)
         self.g_ui_manager = gtk.UIManager()
         self.m_action_groups = {
-            'Exercises': gtk.ActionGroup('Exercises'),
+            'Practise': gtk.ActionGroup('Practise'),
             'Exit': gtk.ActionGroup('Exit'),
             'NotExit': gtk.ActionGroup('NotExit'),
         }
@@ -284,14 +300,14 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
             loc, filename = self.default_learning_tree
         try:
             if loc == 'solfege':
-                self.m_tree.load(filename)
+                self.m_tree.load(os.path.join("learningtrees", filename))
             else:
                 assert loc == 'user'
-                self.m_tree.load(os.path.join(filesystem.app_data(), 'learningtrees', filename))
+                self.m_tree.load(os.path.join(filesystem.user_data(), 'learningtrees', filename))
         except IOError:
             if splash_win:
                 splash_win.hide()
-            self.m_tree.load(self.default_learning_tree[1])
+            self.m_tree.load(os.path.join("learningtrees", self.default_learning_tree[1]))
             self.set_list('app/learningtree', self.default_learning_tree)
             gu.dialog_ok(_("Learning tree '%s' not found. Using default tree." % filename))
             if splash_win:
@@ -355,39 +371,26 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
     def create_learning_trees_menu(self):
         if self.m_learning_trees_merge_id:
             self.g_ui_manager.remove_ui(self.m_learning_trees_merge_id)
-        try:
-            v = os.listdir(os.path.join(filesystem.app_data(), "learningtrees"))
-        except OSError:
-            v = []
-        if not self.m_app.m_options.debug and not v:
-            return
-        actions = [('LearningTreesMenu', None, _('Learning tree')),
-            ('DefaultTree', None, _("Default learning tree"),
-            None, None,
-           lambda o: self.change_learning_tree(self.default_learning_tree)
-            )]
-        if self.m_app.m_options.debug:
-            actions = [('LearningTreesMenu', None, _('Learning tree')),
-                ('DefaultTree', None, _("Default learning tree"),
-                 None, None,
-                 lambda o: self.change_learning_tree(self.default_learning_tree)
-                ),
-                ('DebuggingTree', None, _("Debugging"),
-                 None, None,
-                 lambda o: self.change_learning_tree(self.debug_learning_tree)
-                )]
+        trees = learning_tree_editor.get_learning_tree_list(self.m_app.m_options.debug)
+        actions = [('LearningTreesMenu', None, _('Learning tree'))]
         s = """
 <menubar name='Menubar'>
  <menu action='FileMenu'>
-  <menu action='LearningTreesMenu' position='top'>
-    <menuitem action='DefaultTree'/>"""
-        if self.m_app.m_options.debug:
-            s += "<menuitem action='DebuggingTree'/>"
-        for tree_fn in v:
-            s += "<menuitem action='LTree-%s'/>" % tree_fn
-            actions.append(('LTree-%s' % tree_fn, None,
-                    tree_fn, None, None,
-                    lambda o, t=tree_fn: self.change_learning_tree(('user', t))))
+  <menu action='LearningTreesMenu' position='top'>\n"""
+        old_cls = trees[0][0]
+        for cls, fn, fullpath in trees:
+            if old_cls != cls:
+                s += "<separator/>\n"
+                old_cls = cls
+            t = learning_tree_editor.LearningTree(self.m_app.lessonfile_manager)
+            if cls == 'solfege':
+                t.read_file(os.path.join("learningtrees", fn))
+            else:
+                t.read_file(os.path.join(os.path.join(filesystem.user_data(), 'learningtrees', fn)))
+            s += "  <menuitem action='%s:%s'/>\n" % (cls, fn)
+            actions.append(('%s:%s' % (cls, fn), None, _(t.m_title),
+                None, None,
+                lambda o, c=cls, f=fn: self.change_learning_tree((c, f))))
         s += "</menu></menu></menubar>"
         self.m_action_groups['NotExit'].add_actions(actions)
         self.m_learning_trees_merge_id = self.g_ui_manager.add_ui_from_string(s)
@@ -403,57 +406,42 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         if self.m_practise_and_test_merge_id:
             self.g_ui_manager.remove_ui(self.m_practise_and_test_merge_id)
             self.g_ui_manager.ensure_update()
-        # This method can be called more than once. So we must first
-        # remove any actions from the last time it was called.
-        for action in self.m_action_groups['Exercises'].list_actions():
-            self.m_action_groups['Exercises'].remove_action(action)
-        # We use this counter to make sure all Action names are unique.
-        # This to avoid warnings if a toplevel menu and a submenu has
-        # the same name
-        id_counter = 0
         s = "<menubar name='Menubar'>"
         tests = "<menu action='TestsMenu'>"
         for menu in self.m_tree.m_menus:
-            id_counter += 1
-            self.m_action_groups['Exercises'].add_action(
-                gtk.Action("%s%i" % (menu.name, id_counter),
-                           _(menu.name), None, None))
-            s += "<menu action='%s%i'>" % (escape(menu.name), id_counter)
+            self.m_action_groups['NotExit'].add_action(
+                gtk.Action(menu.name, _(menu.name), None, None))
+            s += "<menu action='%s'>" % escape(menu.name)
             # The following line might look strange here, but it makes sure
             # that the menus on the menu bar stay at the correct order after
             # we save the learning tree in the learning tree editor.
-            self.g_ui_manager.add_ui_from_string("<menubar name='Menubar'><menu action='%s%i'/></menubar>" % (escape(menu.name), id_counter))
+            self.g_ui_manager.add_ui_from_string("<menubar name='Menubar'><menu action='%s'/></menubar>" % escape(menu.name))
             for topic in menu.children:
-                id_counter += 1
-                s += "<menu action='%s%i'>" % (escape(topic['name']), id_counter)
-                tests += "<menu action='TEST_%s%i'>" % (escape(topic['name']), id_counter)
-                actions.append(("%s%i" % (topic['name'], id_counter),
-                                None, _(topic['name'])))
-                actions.append(('TEST_%s%i' % (topic['name'], id_counter),
-                               None, _(topic['name'])))
+                s += "<menu action='%s'>" % escape(topic['name'])
+                tests += "<menu action='TEST_%s'>" % escape(topic['name'])
+                actions.append((topic['name'], None, _(topic['name'])))
+                actions.append(('TEST_%s' % topic['name'], None, _(topic['name'])))
                 for lesson_id in topic.children:
                     if self.m_tree.m_visibilities[lesson_id] > self.m_tree.m_visibility:
                         continue
                     menutitle = self.m_app.lessonfile_manager.m_uiddb[lesson_id]['header']['title']
-                    id_counter += 1
                     actions.append((
-                        "%s%i" % (lesson_id, id_counter),
-                        None, menutitle, None, None,
+                        lesson_id, None, menutitle, None, None,
                         lambda o, e=lesson_id: self.m_app.practise_lesson_id(e)))
                     if self.m_app.lessonfile_manager.is_test_passed(lesson_id):
                         menutitle = _("%s (passed)") % menutitle
                     if self.m_app.lessonfile_manager.get(lesson_id, 'test'):
                         actions.append((
-                            'TEST_%s%s' % (lesson_id, id_counter), None,
+                            'TEST_%s' % lesson_id, None,
                             menutitle,
                             None, None,
                             lambda o, e=lesson_id: self.m_app.test_lesson_id(e)))
-                        tests += "<menuitem action='TEST_%s%i' />" % (escape(lesson_id), id_counter)
-                    s += "<menuitem action='%s%i' />" % (escape(lesson_id), id_counter)
+                        tests += "<menuitem action='TEST_%s' />" % escape(lesson_id)
+                    s += "<menuitem action='%s' />" % escape(lesson_id)
                 s += "</menu>"
                 tests += "</menu>"
             s += "</menu>"
-        self.m_action_groups['Exercises'].add_actions(actions)
+        self.m_action_groups['NotExit'].add_actions(actions)
         tests += "</menu>"
         s += tests
         s += "</menubar>"
@@ -509,8 +497,10 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
                     'pagename': 'SITS-Incoming/SearchBugs',
                     'q': 'SITS-Incoming/"Submitter: %s"' % utils.mangle_email(self.g_email.get_text()),
                 })
-            import webbrowser
-            webbrowser.open_new("http://www.solfege.org?%s" % params)
+            try:
+                webbrowser.open_new("http://www.solfege.org?%s" % params)
+            except Exception, e:
+                self.display_error_message2(_("Error opening web browser"), str(e))
     def display_exception_message(self, exception, lessonfile=None):
         """Call this function only inside an except clause."""
         from exceptiondialog import ExceptionDialog
@@ -603,10 +593,9 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         a.set_logo(pixbuf)
         a.set_name("GNU Solfege")
         a.set_website("http://www.solfege.org")
-        a.set_version(configureoutput.VERSION_STRING)
+        a.set_version(buildinfo.VERSION_STRING)
         a.set_copyright("\n".join((solfege_copyright, warranty)))
         a.set_authors(["Tom Cato Amundsen",
-              'Oliver Vollmer %s' % _("(toolbar icons)"),
               'Giovanni Chierico %s' % _("(some lessonfiles)"),
               'Michael Becker %s' % _("(some lessonfiles)"),
               'Joe Lee %s' % _("(sound code for the MS Windows port)"),
@@ -673,10 +662,11 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
 
                 if splash_win:
                     splash_win.show()
-        # MIGRATION 3.9.3 when we added langenviron.bat
-        if sys.platform == 'win32' and self.m_app.win32_get_langenviron() != self.get_string('app/lc_messages'):
+        # MIGRATION 3.9.3 when we added langenviron.bat and in 3.11
+        # we migrated to langenviron.txt because we does not use cmd.exe
+        if sys.platform == 'win32' and winlang.win32_get_langenviron() != self.get_string('app/lc_messages'):
             gu.dialog_ok(_("Migrated old language setup. You might have to restart the program all translated messages to show up."))
-            self.m_app.win32_put_langenviron(self.get_string('app/lc_messages'))
+            winlang.win32_put_langenviron(self.get_string('app/lc_messages'))
         if self.m_app.lessonfile_manager.m_discards:
             if splash_win:
                 splash_win.hide()
@@ -766,9 +756,19 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         """
         if not self.g_help_browser:
             self.g_help_browser = HelpBrowser(self.m_app)
-        self.g_help_browser.show()
-        self.g_help_browser.present()
-        self.g_help_browser.show_docfile(fn, anchor)
+        if self.get_bool('gui/web_browser_as_help_browser'):
+            for lang in self.g_help_browser.g_docviewer.m_language, "C":
+                filename = os.path.join(os.getcwdu(), u"help", self.g_help_browser.g_docviewer.m_language, fn)
+                if os.path.isfile(fn):
+                    break
+            try:
+                webbrowser.open(filename)
+            except Exception, e:
+                self.display_error_message2(_("Error opening web browser"), str(e))
+        else:
+            self.g_help_browser.show()
+            self.g_help_browser.present()
+            self.g_help_browser.show_docfile(fn, anchor)
     def display_docfile_in_mainwin(self, fn, anchor=None):
         self._show_docviewer()
         self.box_dict['docviewer'].read_docfile(fn, anchor)
@@ -815,7 +815,6 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
             gu.dialog_ok(_("The '%s' exercise module does not support test yet." % self.m_viewer))
             return
         self.m_action_groups['NotExit'].set_sensitive(False)
-        self.m_action_groups['Exercises'].set_sensitive(False)
         self.g = self.box_dict[self.m_viewer].g_notebook.get_nth_page(0)
         self.box_dict[self.m_viewer].g_notebook.get_nth_page(0).reparent(self.main_box)
         self.box_dict[self.m_viewer].g_notebook.hide()
@@ -823,7 +822,6 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
     def exit_test_mode(self):
         self.m_app.m_test_mode = False
         self.m_action_groups['NotExit'].set_sensitive(True)
-        self.m_action_groups['Exercises'].set_sensitive(True)
         box = gtk.VBox()
         self.box_dict[self.m_viewer].g_notebook.insert_page(box, gtk.Label(_("Practise")), 0)
         self.g.reparent(box)
