@@ -1,7 +1,9 @@
 # Solfege - free ear training software
+# vim: set fileencoding=utf-8 :
 # Copyright (C) 2007, 2008 Tom Cato Amundsen
 # License is GPL, see file COPYING
 
+import gtk
 import unittest
 import time
 
@@ -409,6 +411,60 @@ class TestCmdline(unittest.TestCase):
             question['music'].play(self.p, question)
             time.sleep(3)
 
+class TestCSound(unittest.TestCase):
+    def test_CSound(self):
+        self.p = QuestionsLessonfile()
+        self.p.parse_string('''
+header {
+    lesson_id = "csound-intonation-M3-5cent"
+    module = idbyname
+    help = "idbyname-intonation"
+    title = "Test "
+}
+
+question {
+ name = "III temperature"
+ set=0
+ csound_sco = """
+f1     0    4096 10   1  ; use GEN10 to compute a sine wave
+; p1  p2    p3  p4     p5    p6     p7
+;ins  strt  dur amp    freq  attack release
+  i1  0     1   10000  440   0.2    0.08
+  i1  +     1   10000  660   0.08    0.08
+e
+ """
+ music = csound(load("lesson-files/share/sinus-ad.orc"), csound_sco)
+}''')
+        question = self.p.m_questions[0]
+        if cfg.get_bool('testing/may_play_sound'):
+            question['music'].play(self.p, question)
+
+
+class TestMma(unittest.TestCase):
+    def test_1arg(self):
+        s = """
+        question {
+            music = mma("1 Cmaj7")
+        }
+        """
+        p = QuestionsLessonfile()
+        p.parse_string(s)
+        q = p.m_questions[0]
+        if cfg.get_bool('testing/may_play_sound'):
+            q['music'].play(p, q)
+    def test_2arg(self):
+        s = """
+        question {
+            music = mma("ballad", "1 Cmaj7")
+        }
+        """
+        p = QuestionsLessonfile()
+        p.parse_string(s)
+        q = p.m_questions[0]
+        if cfg.get_bool('testing/may_play_sound'):
+            q['music'].play(p, q)
+
+
 class TestLessonfileHeader(I18nSetup):
     def test_untranslated_variable(self):
         self.p = QuestionsLessonfile()
@@ -534,6 +590,53 @@ class TestLessonfileMisc(unittest.TestCase):
         p = IdPropertyLessonfile()
         p.parse_string(s)
         self.assertEquals(p.header.qprops, [u'name'])
+    def test_nrandom(self):
+        self._worker_Xrandom("nrandom")
+    def test_prandom(self):
+        self._worker_Xrandom("prandom")
+    def _worker_Xrandom(self, rfunc):
+        styles = ["8beat", "ballad", "rock"]
+        s = """
+        styles = %s
+        question {
+            style = %s("8beat", "ballad")
+        }
+        question {
+            style = %s(styles)
+        }
+        """ % (", ".join(['"%s"' % x for x in styles]), rfunc, rfunc)
+        p = QuestionsLessonfile()
+        p.parse_string(s)
+        for idx in range(2):
+            st = p.m_questions[idx]['style']
+            for x in range(20):
+                self.assert_(str(st) in styles)
+                self.assert_(unicode(st) in styles)
+                st.randomize()
+            # This test only works with nrandom, because prandom will
+            # return a possible different value every time.
+            if rfunc == 'nrandom':
+                l = gtk.Label(st)
+                self.assert_(l.get_text() == unicode(st), "%s and gtk does not cooperate." % rfunc)
+
+class TestLabelObject(unittest.TestCase):
+    def test_construct(self):
+        l = LabelObject("pangomarkup", "abc")
+        self.assertEquals(l.cval, ("pangomarkup", "abc"))
+    def test_rn_markup(self):
+        for s, v in (
+            ("Imaj7-", [('I', 'maj', '7', '-')]),
+            ("bIIIm7 -", [('bIII', 'm', '7', ' -')]),
+            ("Imaj7-IIm7-V9-Imaj7", [('I', 'maj', '7', '-'),
+                                     ('II', 'm', '7', '-'),
+                                     ('V', '', '9', '-'),
+                                     ('I', 'maj', '7', '')]),
+            ("Imaj7", [('I', 'maj', '7', '')]),
+            ("iim9b5+13", [('ii', 'm', '9b5+13', '')]),
+            ("Imaj7-V7", [('I', 'maj', '7', '-'), ('V', '', '7', '')]),
+            ("Imaj7  -IIm9", [('I', 'maj', '7', '  -'), ('II', 'm', '9', '')]),
+                ):
+            self.assertEquals(rn_markup_tokenizer(s), v, (rn_markup_tokenizer(s), v))
 
 
 suite = unittest.makeSuite(TestParser)
@@ -547,7 +650,10 @@ suite.addTest(unittest.makeSuite(TestPercussion))
 suite.addTest(unittest.makeSuite(TestWavfile))
 suite.addTest(unittest.makeSuite(TestMidifile))
 suite.addTest(unittest.makeSuite(TestCmdline))
+suite.addTest(unittest.makeSuite(TestCSound))
+suite.addTest(unittest.makeSuite(TestMma))
 suite.addTest(unittest.makeSuite(TestLessonfileHeader))
 suite.addTest(unittest.makeSuite(TestLessonfileQuestion))
 suite.addTest(unittest.makeSuite(TestLessonfileMisc))
 suite.addTest(unittest.makeSuite(TestErrorHandling))
+suite.addTest(unittest.makeSuite(TestLabelObject))
