@@ -6,9 +6,6 @@ build() {
   sed -e "s/type=external-midiplayer/type=sequencer-device/" tmp.cfg > default.config
   rm tmp.cfg
 
-  cp solfege.bat solfegedebug.bat.tmp
-  echo "pause">> solfegedebug.bat.tmp
-  sed -e "s/python.exe solfege/python.exe solfege --debug/" solfegedebug.bat.tmp > solfegedebug.bat
   ./configure PYTHON=/c/Python25/python.exe --disable-pygtk-test --enable-winmidi
   make skipmanual=yes PYTHON_INCLUDES=-I/c/Python25/include
   make winbuild
@@ -22,8 +19,9 @@ prepare_installer() {
   cp -a ../gtk-kopi win32/GTK
   (cd win32/python/lib/site-packages && tar zxf ../../../../../pygtk-stuff.tgz)
   (cp soundcard/winmidi.pyd win32/share/solfege/soundcard)
-  cp solfege.bat win32/bin/
+  cp win32-start-solfege.pyw win32/bin
   cp solfegedebug.bat win32/bin/
+  rm win32/bin/lessonfile_editor
   cp README.txt INSTALL.win32.txt INSTALL.txt AUTHORS.txt COPYING.txt win32
   (cd win32 && find -name *.pyc | xargs rm)
   python tools/trim_win32_installer.py
